

import os
import subprocess


def get_command_line_arguments():
    """
    Returns a list of files that have been provided as a command line argument
    :return: list of files
    """

    # Parse and assert command line options
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--file-name", default=None,
                        help="Name of the job you want to create or modify")
    parser.add_argument("--set-time", default=None, type=int,
                        help="Set slurm time, in minutes")
    parser.add_argument("--set-time-seconds", default=None, type=int,
                        help="Set slurm time, in minutes")
    parser.add_argument("--set-core", default=1, type=int,
                        help="Set number of cores to be used for the job")
    parser.add_argument("--set-core-per-node", default=48, type=int,
                        help="Set number of cores to be used for the job")
    parser.add_argument("-j", "--job-name", default=None,
                        help="Name of the job you want to create or modify")
    parser.add_argument("--set-queue", default="debug",
                        help="Set the queue name")
    parser.add_argument("-w", "--set-workload",
                        default=None, nargs='+',
                        help="list of commands that will be executed in the script")
    parser.add_argument("-s", "--scheduler",
                        default="slurm",
                        help="scheduler used (slurm/lsf)")

    args = parser.parse_args()
    if args.set_time_seconds is not None:
        if args.set_time is None:
            # Covert time ensuring to request at least 1 min.
            args.set_time = int(args.set_time_seconds / 60.) + 1
        else:
            parser.error("Just one time unity can be specified")
    if args.file_name is None:
        parser.error("The job filename has not been specified")
    return args


def create_job_slurm(args):
    
    file_name = args.file_name
    time = args.set_time
    cores = args.set_core
    name = args.job_name
    cores_per_node = args.set_core_per_node
    queue = args.set_queue
    workload = args.set_workload


    file = ["#!/bin/bash \n",
            "############################################################################### \n",
            "#SBATCH --output "+str(name)+".out \n#SBATCH --error "+str(name)+".err \n"
            "#SBATCH --constraint=perfparanoid \n"]

    if cores is not None:
        file.append("#SBATCH --ntasks " + str(cores) + "\n")
    if cores_per_node is not None:
        file.append("#SBATCH --ntasks-per-node " + str(cores_per_node) + "\n")
    if time is not None and not 0:
        file.append("#SBATCH --time " + str(time) + "\n")
    if name is not None:
        file.append("#SBATCH -J " + name + "\n")
    if queue is not None and not len(queue) == 0:
        file.append("#SBATCH --qos=" + queue + "\n")

    if workload is not None:
        file.append("\n")
        for work in workload:
            file.append(str(work) + "")

    with open(file_name, "w") as slurm_job:
        for line in file:
            slurm_job.write(line)


def create_job_lsf(args):
    file = ["#!/bin/bash \n",
            "############################################################################### \n",
            "#BSUB -o "+str(name)+".out \n#BSUB -e "+str(name)+".err \n"
            "#BSUB --constraint=perfparanoid \n"]
    file_name = args.file_name
    time = args.set_time
    cores = args.set_core
    name = args.job_name
    cores_per_node = args.set_core_per_node
    queue = args.set_queue
    workload = args.set_workload

    if cores is not None:
        file.append("#BSUB -n " + str(cores) + "\n")
    if time is not None and not 0:
        file.append("#BSUB -W " + str(time) + "\n")
    if name is not None:
        file.append("#BSUB-J " + name + "\n")
    if queue is not None and not len(queue) == 0:
        file.append("#BSUB -q " + queue + "\n")

    if workload is not None:
        file.append("\n")
        for work in workload:
            file.append(str(work) + "")

    with open(file_name, "w") as lsf_job:
        for line in file:
            lsf_job.write(line)


def modify_job(args):
    file_name = args.file_name
    time = args.set_time
    cores = args.set_core
    name = args.job_name
    cores_per_node = args.set_core_per_node
    queue = args.set_queue
    workload = args.set_workload

    # if cores is not None:
    #  subprocess.call(["sed -i 's|.*SBATCH --ntasks .*|SBATCH --ntasks '+cores+file_name"], shell=True)

    # if cores_per_node is not None:
    #  file.append("#SBATCH --ntasks-per-node " + str(cores_per_node) + "\n")
    # if time is not None:
    #    file.append("#SBATCH --time " + str(time) + "\n")
    # if name is not None:
    #    file.append("#SBATCH -J " + name + "\n")
    # if queue is not None:
    # file.append("#SBATCH --qos=" + queue + "\n")

    # if workload is not None:


if __name__ == "__main__":
    # Get files from command line
    args = get_command_line_arguments()
    if args.scheduler == "slurm":
        args.file_name = args.file_name if args.file_name.count(".slurm") else args.file_name + ".slurm"
    elif args.scheduler == "lsf":
        args.file_name = args.file_name if args.file_name.count(".lsf") else args.file_name + ".lsf"

    if os.path.exists(str(args.file_name)):
        os.remove(str(args.file_name))

    if args.scheduler == "slurm":
        create_job_slurm(args)
    elif args.scheduler == "lsf":
        create_job_lsf(args)

